// mwa.products.NLBean.java
//
// Copyright 1997, Mark Watson.
//
// This software is Copyright 1998 by Mark Watson.
// This software may be freely used for any purpose
// and may be modified for any purpose as long as
// you agree to the following:
//
// 1. Any bug fixes or enhancements that you make must be emailed
//    back to Mark Watson (markw@markwatson.com) for possible
//    inclusion in the source code base that is available at
//    the web site www.markwatson.com (credit will be given
//    to contributers).
//
// 2. This copyright notice must be kept with the source code
//    and duplicated if you distribute a modified verison of
//    this software in binary form.
//
// 3. You will not re-distribute the source code. Rather, any
//    improvements to this program will be returned to Mark
//    Watson so that there is always one source base at
//    the web site www.markwatson.com (you may freely
//    distribute compiled derived versions of this program
//    without any restrictions as long as this entire
//    copyright notice is reproduced with the application
//    (e.g., in the 'help' or 'about' dialog box).
//
// 4. This software may not be used for any illegal purpose.
//
// 5. You take all responsibility for the use of this software:
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// Open Source contributers:
//
//   Name                        Email (optional)
//   ----                        ----------------
//



package com.markwatson.nlbean;

import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.text.*;
import java.util.*;
import java.beans.*;
import java.util.Vector;

public class NLBean extends Panel implements Serializable, Runnable {

    protected NLEngineLocal engine = null;

    private boolean applicationFlag = false;

    public NLBean(boolean app_flag) {
        super();
        applicationFlag = app_flag;
        reset();
    }

    public NLBean() {
        super();
        applicationFlag = false;
        reset();
    }


    private void writeObject(java.io.ObjectOutputStream s)
        throws java.io.IOException {
        System.out.println("Beginning writeObject");
        s.defaultWriteObject();
        System.out.println("Ending writeObject");
    }

    private void resetExamples() {
        choiceChanged = false;
        choice.removeAll();
        choice.addItem("Examples                                        ");
        if (example10!=null) choice.insert(example10, 1);
        if (example9!=null) choice.insert(example9, 1);
        if (example8!=null) choice.insert(example8, 1);
        if (example7!=null) choice.insert(example7, 1);
        if (example6!=null) choice.insert(example6, 1);
        if (example5!=null) choice.insert(example5, 1);
        if (example4!=null) choice.insert(example4, 1);
        if (example3!=null) choice.insert(example3, 1);
        if (example2!=null) choice.insert(example2, 1);
        if (example1!=null) choice.insert(example1, 1);
    }

    private void resetSynonyms() {
        engine.clearSynonyms();
        if (synonym1!=null) addSynonym(synonym1);
        if (synonym2!=null) addSynonym(synonym2);
        if (synonym3!=null) addSynonym(synonym3);
        if (synonym4!=null) addSynonym(synonym4);
        if (synonym5!=null) addSynonym(synonym5);
        if (synonym6!=null) addSynonym(synonym6);
        if (synonym7!=null) addSynonym(synonym7);
        if (synonym8!=null) addSynonym(synonym8);
        if (synonym9!=null) addSynonym(synonym9);
        if (synonym10!=null) addSynonym(synonym10);
    }

    private void readObject(java.io.ObjectInputStream s)
        throws java.lang.ClassNotFoundException,
        java.io.IOException {
        System.out.println("Beginning readObject");
        s.defaultReadObject();

        resetSynonyms();
        resetExamples();
	    
        System.out.println("  create and start workThread...");
        workThread = new Thread(this);
        workThread.start();
        System.out.println("Ending readObject");
    }

    //              Set up USER INTERFACE:

    private void reset() {

        engine = new NLEngineLocal();
        resetDB();
        AWTsetup();
    }

    private void resetDB() {
    engine.clearDB();
    
    if (databaseName1!=null) {
      engine.addDB(databaseName1, userName1, password1, tableList1);
    }
    if (databaseName2!=null) {
      engine.addDB(databaseName2, userName2, password2, tableList2);
    }
    if (databaseName3!=null) {
      engine.addDB(databaseName3, userName3, password3, tableList3);
    }
    
    engine.initDB();
    }
    
    private void AWTsetup() {

        setFont(new Font("Dialog", Font.PLAIN, 12));
        setLayout(null);

        Label l1 = new Label("Natural Language Database Access");
        l1.setFont(new Font("Dialog", Font.BOLD, 32));
        add(l1);
        l1.setBounds(3, 5, 560, 38);

        Button q_button = new Button("Query:");
        q_button.addMouseListener(new MouseQuery());
        q_button.setFont(new Font("Dialog", Font.BOLD, 20));
        add(q_button);
        q_button.setBounds(3, 50, 90, 25);

        inputText = new SmartTextField("list Salary where EmpName equals Mark", 64);
        add(inputText);
        inputText.setBounds(97, 50, 480, 25);

        choice = new Choice();
        choice.setFont(new Font("Dialog", Font.BOLD, 12));
        choiceChanged = false;
        choice.addItem("Examples                                              ");
        if (example10!=null) choice.insert(example10, 1);
        if (example9!=null) choice.insert(example9, 1);
        if (example8!=null) choice.insert(example8, 1);
        if (example7!=null) choice.insert(example7, 1);
        if (example6!=null) choice.insert(example6, 1);
        if (example5!=null) choice.insert(example5, 1);
        if (example4!=null) choice.insert(example4, 1);
        if (example3!=null) choice.insert(example3, 1);
        if (example2!=null) choice.insert(example2, 1);
        if (example1!=null) choice.insert(example1, 1);
        choice.addItemListener(new ChoiceListener());
        add(choice);
        choice.setBounds(3, 80, 575, 25);

        Label label23 = new Label("Generated SQL:");
        label23.setFont(new Font("Dialog", Font.BOLD, 12));
        add(label23);
        label23.setBounds(3, 110, 90, 25);
        sqlText = new TextArea("",4,80,TextArea.SCROLLBARS_NONE);
        sqlText.setEditable(false);
        add(sqlText);
        sqlText.setBounds(105, 110, 470, 75);

        Label credits =
          new Label("Copyright 1998 by Mark Watson.   Open Source Java distribution at www.markwatson.com");
        credits.setFont(new Font("Dialog", Font.BOLD, 12));
        add(credits);
        credits.setBounds(3, 200, 570, 20);
        setupHelper();

        usingAWT = true;  // enables list handling code in method run()

    }

    private void setupHelper() {

        if (loadSynonyms) {
            if (synonym1!=null) addSynonym(synonym1);
            if (synonym2!=null) addSynonym(synonym2);
            if (synonym3!=null) addSynonym(synonym3);
            if (synonym4!=null) addSynonym(synonym4);
            if (synonym5!=null) addSynonym(synonym5);
            if (synonym6!=null) addSynonym(synonym6);
            if (synonym7!=null) addSynonym(synonym7);
            if (synonym8!=null) addSynonym(synonym8);
            if (synonym9!=null) addSynonym(synonym9);
            if (synonym10!=null) addSynonym(synonym10);
        }

        workThread = new Thread(this);
        workThread.start();

        System.out.println("setupHelper()  4");
    }

    synchronized private boolean goodWord(String s) {
        return engine.goodWord(s);
    }

    // Suggest spelling corrections, or other words:

    synchronized private String[] suggestedWords(String a_word) {
        return engine.suggestedWords(a_word);
    }

    synchronized private String inText(String new_val, boolean set_flag) {
        if (set_flag) {
            inputText.setText(new_val);
            return "";
        }
        return inputText.getText();
    }

    synchronized private void query() {
        System.out.println("Entering query()");

        sqlText.setText("");

        String a_query = inText("", false);
        System.out.println("query(): a_query=" + a_query);
        String sql_query=null;
        if (a_query.startsWith("SELECT") || a_query.startsWith("select")) {
            sql_query = a_query;
        } else {
            engine.parse(a_query);
            sql_query = engine.getSQL();
        }

        if (sql_query==null) {
            System.out.println("No SQL for " + a_query);
            return;
        }
        System.out.println("SQL query: " + sql_query);
        sqlText.setText(sql_query);
    }

    private boolean usingAWT = false;

    public void run() {
        while (true) {
            try {

                try {
                    workThread.sleep(200); // sleep for 1/5 second
                    if (process) {
                        query();
                        setProcess(false);
                    }
                } catch (Exception e) {
                    System.out.println("Error: " + e);
                    e.printStackTrace ();
                    setProcess(false);
                }
            } catch (Exception run_e) {
                System.out.println("run() exception: " + run_e);
            }
        }
    }


    private boolean loadSynonyms = true;
    private boolean loadDB = true;

    private void addSynonym(String def_string) {
        int pos = def_string.indexOf("=");
        String description = def_string.substring(0,pos);
        String column = def_string.substring(pos+1);
        if (engine!=null) {
            engine.addSynonym(column, description);
        }
    }

    // Use an inner classes for mouse event handling:
    class MouseQuery extends MouseAdapter implements Serializable {
        // The magic: access the public query method in the
        // containing class:
        synchronized public void mouseReleased(MouseEvent mevt) {
            setProcess(true);
        }
    }

    class MouseHalt extends MouseAdapter implements Serializable {
        // The magic: access the public query method in the
        // containing class:
        synchronized public void mouseReleased(MouseEvent mevt) {
            setProcess(false);
        }
    }

    private Choice choice;
    transient private boolean choiceChanged = false;

    class ChoiceListener implements ItemListener {
        public void itemStateChanged(ItemEvent ie) {
            System.out.println("choice menu: " + ie.paramString());
            System.out.println("choice menu: " + (String)ie.getItem());
            String sel = (String)ie.getItem();
            if (sel.equals("Examples"))  return;
            inText(sel, true);
            if (choiceChanged==false) {
                choice.remove(0);
                choiceChanged=true;
            }
            System.out.println("choice menu: <returning>");
        }
    }

    protected String suggested_words[] = null;

    public static  void main(String args[]) {
        AppFrame f = new AppFrame();
        NLBean sc = new NLBean(true);
        f.add(sc);
        sc.setBounds(0, 5, 580, 230);
        f.setTitle("NLBean version 1.2");
        f.pack();
        f.show();
    }

    private SmartTextField inputText;
    private TextArea sqlText;
    private transient Thread workThread;
    private boolean process;

    private List list1, list2, list3;
    private int list1_last_selection, list2_last_selection,  list3_last_selection;

    synchronized public void setProcess(boolean p) { 
        process=p;
        System.out.println("setProcess(" + p + ")");
    }

    // properties for specifying a few sample natural language
    // queries to place at the beginning of the 'Example'
    // 'choice' control.  NOTE: this program automatically
    // adds additional examples that it generates from examining
    // the field names of the database tables.

    private String example1 = "list email address where name equals Mark";
    private String example2 = "list salary where employee name equals Mark";
    private String example3 = "list name, phone number, and email address where name equals Mark";
    private String example4 = "list employee name, salary, and hire date where hire date is after January 10, 1993";
    private String example5 = "list salary where hire date is after January 1, 1993 or employee name equals Carol";
    private String example6 = "list product name where cost is less than $20";
    private String example7 = null;
    private String example8 = null;
    private String example9 = null;
    private String example10 = null;
    public void setExample1(String s) { example1=s; resetExamples(); }
    public void setExample2(String s) { example2=s; resetExamples(); }
    public void setExample3(String s) { example3=s; resetExamples(); }
    public void setExample4(String s) { example4=s; resetExamples(); }
    public void setExample5(String s) { example5=s; resetExamples(); }
    public void setExample6(String s) { example6=s; resetExamples(); }
    public void setExample7(String s) { example7=s; resetExamples(); }
    public void setExample8(String s) { example8=s; resetExamples(); }
    public void setExample9(String s) { example9=s; resetExamples(); }
    public void setExample10(String s) { example10=s; resetExamples(); }
    public String getExample1() { return example1; }
    public String getExample2() { return example2; }
    public String getExample3() { return example3; }
    public String getExample4() { return example4; }
    public String getExample5() { return example5; }
    public String getExample6() { return example6; }
    public String getExample7() { return example7; }
    public String getExample8() { return example8; }
    public String getExample9() { return example9; }
    public String getExample10() { return example10; }

    // Set up properties for synonyms:
    private String synonym1 = "employee name=EmpName";
    private String synonym2 = "hire date=HireDate";
    private String synonym3 = "phone number=PhoneNumber";
    private String synonym4 = "email address=Email";
    private String synonym5 = "product name=productname";
    private String synonym6 = "products=productname";
    private String synonym7 = "product=productname";
    private String synonym8 = null;
    private String synonym9 = null;
    private String synonym10 = null;
    public void setSynonym1(String s) { synonym1=s; resetSynonyms(); }
    public void setSynonym2(String s) { synonym2=s; resetSynonyms(); }
    public void setSynonym3(String s) { synonym3=s; resetSynonyms(); }
    public void setSynonym4(String s) { synonym4=s; resetSynonyms(); }
    public void setSynonym5(String s) { synonym5=s; resetSynonyms(); }
    public void setSynonym6(String s) { synonym6=s; resetSynonyms(); }
    public void setSynonym7(String s) { synonym7=s; resetSynonyms(); }
    public void setSynonym8(String s) { synonym8=s; resetSynonyms(); }
    public void setSynonym9(String s) { synonym9=s; resetSynonyms(); }
    public void setSynonym10(String s) { synonym10=s; resetSynonyms(); }
    public String getSynonym1() { return synonym1; }
    public String getSynonym2() { return synonym2; }
    public String getSynonym3() { return synonym3; }
    public String getSynonym4() { return synonym4; }
    public String getSynonym5() { return synonym5; }
    public String getSynonym6() { return synonym6; }
    public String getSynonym7() { return synonym7; }
    public String getSynonym8() { return synonym8; }
    public String getSynonym9() { return synonym9; }
    public String getSynonym10() { return synonym10; }

    // for four properties each for up to three databases:

    //    database #1
    private String databaseName1 = "product_test1";
    private String userName1 = "Admin";
    private String password1 = "sammy";
    private String tableList1= "NameTable(Name,Email,PhoneNumber,Misc);Employees(EmpName,HireDate,Salary);Products(productname,cost)";

    public void setDatabaseName1(String n) {
        databaseName1 = n;
        resetDB();
    }
    public String getDatabaseName1() { return databaseName1; }
    public void setUserName1(String n) { userName1 = n; }
    public String getUserName1() { return userName1; }
    public void setPassword1(String n) { password1 = n; }
    public String getPassword1() { return password1; }
    public void setTableList1(String n) { tableList1 = n; }
    public String getTableList1() { return tableList1; }

    //    database #2
    private String databaseName2 = null; // "LocalDatabase";
    private String userName2 = "";
    private String password2 = "";
    private String tableList2= "";
    public void setDatabaseName2(String n) {
        databaseName2 = n;
        resetDB();
    }
    public String getDatabaseName2() { return databaseName2; }
    public void setUserName2(String n) { userName2 = n; }
    public String getUserName2() { return userName2; }
    public void setPassword2(String n) { password2 = n; }
    public String getPassword2() { return password2; }
    public void setTableList2(String n) { tableList2 = n; }
    public String getTableList2() { return tableList2; }

    //    database #3
    private String databaseName3 = null;
    private String userName3 = "";
    private String password3 = "";
    private String tableList3= "";
    public void setDatabaseName3(String n) {
        databaseName3 = n;
        resetDB();
    }
    public String getDatabaseName3() { return databaseName3; }
    public void setUserName3(String n) { userName3 = n; }
    public String getUserName3() { return userName3; }
    public void setPassword3(String n) { password3 = n; }
    public String getPassword3() { return password3; }
    public void setTableList3(String n) { tableList3 = n; }
    public String getTableList3() { return tableList3; }

    //
    //            Inner class to handle text input field:
    //

    transient protected int startWordPos=0, stopWordPos=0;

    class SmartTextField extends TextField implements Serializable {
        public SmartTextField(String s, int width) {
            super(s, width);
            helper();
        }
        public SmartTextField(String s) {
            super(s);
            helper();
        }
        public SmartTextField() {
            super();
            helper();
        }
        private void helper() {
            addMouseListener(new MouseText());
            addKeyListener(new MyKeyAdapter());
            setEditable(true);
            setFont(new Font("default", Font.PLAIN, 12));
            words = new String[20];
            for (int i=0; i<20; i++) words[i]="";
            num_words=0;
        }

        transient private String words[] = null;
        transient int num_words;

        public String getWord(int charPos) {
            startWordPos=stopWordPos=-1;
            String s = getText();
            // find start of word:
            int start = charPos-1;
            try {
                while (start >= 0 && s.charAt(start)!=' ') {
                    start--;
                }
                start++;
            } catch (Exception e) {}
            // find the end of word:
            int stop=charPos;
            try {
                while (stop < s.length() && s.charAt(stop)!=' ') {
                    stop++;
                }
            } catch (Exception e) {}
            if (start<0) start=0;
            //if (stop>s.length()-1) stop=s.length()-1;
            System.out.println("getWord(): start=" + start + ", stop=" + stop);
            //if (stop > s.length() - 2) stop = s.length() - 1;
            int stp = stop;
            if (stop>my_char_pos) {
                stop=my_char_pos;
                System.out.println("    stop reset to " + stop);
            }
            if (start < stop) {
                startWordPos=start;
                stopWordPos=stp;

                System.out.println("getWord() returning: |" + s.substring(start, stop) + "|");

                return s.substring(start, stop);
            }
            else return "";
        }


        public void setUpperCase(int charPos) {
            String s = getText();
            // find start of word:
            int start = charPos-1;
            try {
                while (start >= 0 && s.charAt(start)!=' ') {
                    start--;
                }
                start++;
            } catch (Exception e) {}
            // find the end of word:
            int stop=charPos;
            try {
                while (stop < s.length() && s.charAt(stop)!=' ') {
                    stop++;
                }
            } catch (Exception e) {}
            if (start<0) start=0;
            if (stop>s.length()) stop=s.length();
            if (stop>my_char_pos) stop=my_char_pos;
            if (start < stop) {
                StringBuffer sb = new StringBuffer(s);
                for (int i=start; i<stop; i++) {
                    char ch = sb.charAt(i);
                    if (ch >= 'a' && ch <= 'z') ch += 'A' - 'a';
                    sb.setCharAt(i, ch);
                }
                setText(new String(sb));
                setCaretPosition(stop);
            }
        }

        transient private int charWidth = -1;
        transient private FontMetrics fm = null;

        public void paint(Graphics g) {
            super.paint(g);
            if (fm==null) {
                fm = g.getFontMetrics();
            }
        }

        private int getPixelWidth(String s) {
            if (fm==null) return 1;
            return fm.stringWidth(s);
        }

        transient private int my_char_pos=0;

        public int getCharPos(int pixel) {
            String s = getText();
            my_char_pos=-1;
            if (getPixelWidth(s) < pixel)  return -1;  // to right of text
            int len = s.length();
            for (int i=1; i<len; i++) {
                if (getPixelWidth(s.substring(0, i)) > pixel - 12)  {
                    my_char_pos=i;
                    return i;
                }
            }
            my_char_pos = len-1;
            return len-1;
        }

        // Handle mouse events over the input text window:
        class MouseText extends MouseAdapter implements Serializable {
            public void mousePressed(MouseEvent mevt) {
                int x = mevt.getX();
                int charPos = getCharPos(x);
                if (charPos<0)  return;
                my_char_pos = 1000; // special for mouse click only!
                String word = getWord(charPos);

                System.out.println("MouseText.mousePressed(): getWord returned: " + word);

                if (word.length() < 1) return;
                System.out.println("x=" + x + ", charPos=" + charPos +
                                   ", word=" + word);

            }
        }

        class MyKeyAdapter extends KeyAdapter implements Serializable {
            public void keyTyped(KeyEvent ke) {
                if (ke.getKeyChar()==KeyEvent.VK_SPACE ||
                    ke.getKeyChar()==KeyEvent.VK_COMMA ||
                    ke.getKeyChar()==KeyEvent.VK_PERIOD)
                    {
                        // Handle new word:
                        //System.out.println("New word. all text:" + getText());
                        int charPos = getCaretPosition();
                        my_char_pos = charPos;
                        System.out.println("** charPos=" + charPos);
                        String word = getWord(charPos);
                        System.out.println("** word =" + word);
                        if (goodWord(word)==false) {
                            setUpperCase(charPos);
                        }
                    }
            }
        }
    }
    //
    //            End of inner class to handle text input field.
}

class AppFrame extends Frame {
    public AppFrame(String s) {
        super(s);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    }
    public AppFrame() {
        super();
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    }
  public void processEvent(AWTEvent evt) { // JDK 1.1
    switch (evt.getID())  {
    case Event.WINDOW_DESTROY:
      System.exit(0);
    default:
      super.processEvent(evt);
    }                    
  }
}    
