// mwa.products.NLEngineLocal.java
//
// Copyright 1997, Mark Watson.
//
// This software is Copyright 1998 by Mark Watson.
// This software may be freely used for any purpose
// and may be modified for any purpose as long as
// you agree to the following:
//
// 1. Any bug fixes or enhancements that you make must be emailed
//    back to Mark Watson (markw@markwatson.com) for possible
//    inclusion in the source code base that is available at
//    the web site www.markwatson.com (credit will be given
//    to contributers).
//
// 2. This copyright notice must be kept with the source code
//    and duplicated if you distribute a modified verison of
//    this software in binary form.
//
// 3. You will not re-distribute the source code. Rather, any
//    improvements to this program will be returned to Mark
//    Watson so that there is always one source base at
//    the web site www.markwatson.com (you may freely
//    distribute compiled derived versions of this program
//    without any restrictions as long as this entire
//    copyright notice is reproduced with the application
//    (e.g., in the 'help' or 'about' dialog box).
//
// 4. This software may not be used for any illegal purpose.
//
// 5. You take all responsibility for the use of this software:
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// Open Source contributers:
//
//   Name                        Email (optional)
//   ----                        ----------------
//



package com.markwatson.nlbean;

import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.text.*;
import java.util.*;
import java.beans.*;
import java.sql.*;
import java.util.Vector;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

public class NLEngineLocal implements Serializable {

  public NLEngineLocal() {

    // Create a DBInfo object and initialize it with the current
    // database properties:
    dbinfo = new DBInfo();

    nlp = new NLP(dbinfo);

    atn = nlp.atn;
  }

  //                     NLP data and methods:

  private ATN atn = null;

  public void parse(String s) {

    // Remove any commas from the input text:
    for (int i=0; i<20; i++) {
      int idx = s.indexOf(",");
      if (idx>-1) {
	s = s.substring(0,idx) + s.substring(idx+1);
      } else {
	break;
      }
    }
    // Remove any periods from the input text:
    for (int i=0; i<10; i++) {
      int idx = s.indexOf(".");
      if (idx>-1) {
	s = s.substring(0,idx) + s.substring(idx+1);
      } else {
	break;
      }
    }

    // remove extra spaces and convert to lower case:
    s = " " + Util.removeExtraSpaces(s).toLowerCase() + " ";

    // before calling the NLP class parse method, we
    // need to replace synonyms:

    numSynReplacements = 0;
    for (int i=0; i<numSynonyms; i++) {
      // allow for multiple uses of the same synonym:
      for (int repeat=0; repeat<4; repeat++) {
	System.out.println("syn desc=|" + synonymDescription[i] + "|");
	int idx = s.indexOf(synonymDescription[i]);
	if (idx>-1) {
	  s = s.substring(0,idx+1) +
            synonymColumnName[i] + 
            s.substring(idx + synonymDescription[i].length() - 1);
	  System.out.println("synonym substitution: " + s);
    syns[numSynReplacements] = synonymColumnName[i];
    origs[numSynReplacements] = synonymDescription[i];
    numSynReplacements++;
	} else {
	  break;
	}
      }
    }

    // remove extra spaces (just to make sure!):
    s = Util.removeExtraSpaces(s);

    nlp.parse(s);
  }

  // See if a word is "known" (this includes table and column names):

  public boolean goodWord(String s) {
    if (atn.wordInLexicon(s.toLowerCase())) return true;
    return false;
  }

  public String getSQL() {
    return nlp.getSQL();
  }

  public String toEnglish(String [] r) {
    return nlp.toEnglish(r, syns, origs, numSynReplacements);
  }

  // Suggest spelling corrections, or other words:

  public String[] suggestedWords(String a_word) {
    String word = a_word.toLowerCase();
    String temp[] = new String[3000];
    int count=0;

    // T.B.D. we should first add column names !!!!
    // (as the most relevent thing here!!!)

    if (word.length() > 4) {
      String t_word = word.substring(0,4);
      for (int i=0; i<atn.numWords; i++) {
        if (atn.allWords[i].startsWith(t_word)) {
          temp[count++]=atn.allWords[i];
        }
      }
    }
    if (word.length() > 3 && count < 100) {
      String t_word = word.substring(0,3);
      for (int i=0; i<atn.numWords; i++) {
        if (atn.allWords[i].startsWith(t_word)) {
          // check to see that we have not
          // already selected this word:
          boolean ok=true;
          for (int k=0; k<count; k++) {
            if (temp[k].equals(atn.allWords[i])) {
              ok=false;
              break;
            }
          }
          if (ok)  temp[count++] = atn.allWords[i];
        }
      }
    }
    if (word.length() > 2 && count < 100) {
      String t_word = word.substring(0,2);
      for (int i=0; i<atn.numWords; i++) {
        if (atn.allWords[i].startsWith(t_word)) {
          // check to see that we have not
          // already selected this word:
          boolean ok=true;
          for (int k=0; k<count; k++) {
            if (temp[k].equals(atn.allWords[i])) {
              ok=false;
              break;
            }
          }
          if (ok)  temp[count++] = atn.allWords[i];
        }
      }
    }
    if (word.length() > 1 && count < 100) {
      String t_word = word.substring(0,1);
      for (int i=0; i<atn.numWords; i++) {
        if (atn.allWords[i].startsWith(t_word)) {
          // check to see that we have not
          // already selected this word:
          boolean ok=true;
          for (int k=0; k<count; k++) {
            if (temp[k].equals(atn.allWords[i])) {
              ok=false;
              break;
            }
          }
          if (ok)  temp[count++] = atn.allWords[i];
        }
      }
    }
    if (count==0)  return null;
    String temp2[] = new String[count];
    for (int i=0; i<count; i++) temp2[i] = temp[i];
    return temp2;
  }

  public  DBInfo dbinfo;
  private NLP nlp;

   public void addDB(String name, String user, String passwd, String tbls) {
    if (numDB < 4) {
      databaseName[numDB] = name;
      userName[numDB] = user;
      password[numDB] = passwd;
      tableList[numDB] = tbls;
      numDB++;
    }
  }

  public void clearDB() {
    // This is called when re-loading a saved JavaBean.
    numDB=0;;
    nlp.clearTables();
    // T.B.D.: remove old table/column names from ATN's hashtables

  }

  public void initDB() {
    for (int d=0; d<numDB; d++) {
      if (databaseName[d]!=null) {
        String tl[] = Util.parseStrings(tableList[d]);
        if (tl!=null) {

          for (int i=0; i<tl.length; i++) {

            int pos_1 = tl[i].indexOf("(");
            if (pos_1 == -1)  continue;
            int pos_2 = tl[i].indexOf(")");
            if (pos_2 == -1) continue;
            String table_name = tl[i].substring(0, pos_1);
            System.out.println("next TABLE: " + table_name);
            String cols[] = Util.parseStringsCommas(tl[i].substring(pos_1, pos_2));
            if (cols != null) {
                atn.addTableName(table_name);
                dbinfo.addTable(table_name, databaseName[d], userName[d], password[d], cols);
                if (atn!=null) {
                  for (int ii=0; ii<cols.length; ii++) {
                    System.out.println("adding column name to lexicon: " + cols[ii]);
                    atn.addColumnName(cols[ii]);
                  }
                } else {
                  System.out.println("null atn");
                }
              }
          }
        }
      }
    }
    dbinfo.debug();
  }


  // Allow developers to define a few synonyms for a particular
  // application:
  private String synonymColumnName[] = new String[11];
  private String synonymDescription[] = new String[11];
  private int numSynonyms = 0;

  // For use in generating English output for queries:
  private String syns[] = new String[11];
  private String origs[] = new String[11];
  private int numSynReplacements=0;

  public void clearSynonyms() {
    numSynonyms = 0;
  }

  public void addSynonym(String column, String description) {
    if (numSynonyms<10) {
      synonymColumnName[numSynonyms] = column;
      synonymDescription[numSynonyms] = " " + description + " ";
      numSynonyms++;
    }
  }

  // for four properties each for up to five databases:
  private String databaseName[] = new String[5];
  private String userName[] = new String[5];
  private String password[] = new String[5];
  private String tableList[] = new String[5];
  private int numDB=0;

}
