// mwa.products.db.NLP.java
//
// Copyright 1997, Mark Watson.
//
// This software is Copyright 1998 by Mark Watson.
// This software may be freely used for any purpose
// and may be modified for any purpose as long as
// you agree to the following:
//
// 1. Any bug fixes or enhancements that you make must be emailed
//    back to Mark Watson (markw@markwatson.com) for possible
//    inclusion in the source code base that is available at
//    the web site www.markwatson.com (credit will be given
//    to contributers).
//
// 2. This copyright notice must be kept with the source code
//    and duplicated if you distribute a modified verison of
//    this software in binary form.
//
// 3. You will not re-distribute the source code. Rather, any
//    improvements to this program will be returned to Mark
//    Watson so that there is always one source base at
//    the web site www.markwatson.com (you may freely
//    distribute compiled derived versions of this program
//    without any restrictions as long as this entire
//    copyright notice is reproduced with the application
//    (e.g., in the 'help' or 'about' dialog box).
//
// 4. This software may not be used for any illegal purpose.
//
// 5. You take all responsibility for the use of this software:
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// Open Source contributers:
//
//   Name                        Email (optional)
//   ----                        ----------------
//



package com.markwatson.nlbean;

public class NLP implements java.io.Serializable {
    public NLP() {
      helper();
      dbinfo=null;
    }
    public NLP(DBInfo dbi) {
      dbinfo=dbi;
      helper();
    }
    private void helper() {
      atn = new ATN();
      //atn.loadLexicon("\\src\\db\\lex.txt");
      atn.loadLexicon("lex");
      atn.dbinfo=dbinfo;
    }

    public ATN atn = null;

    public void clearTables() { dbinfo.num_tables=0; }

    final private int AND=1;
    final private int OR=2;

    public void parse(String s) {

      // parse a new sentence:
      currentWords = Util.parseStrings(s);
      if (currentWords==null) {
        System.out.println("Error parsing: " + s);
        return;
      }

      // Check for a REFINED QUERY that builds on the
      // last query (in which case, we do not want to
      // blank-out the slots from the last parse.
      //
      // Support:
      //   Mode 0:
      //         new query
      //
      //   Mode 1:
      //         and <column name>   -- adds a display column name
      //
      //   Mode 2:
      //         and <condition>     -- adds a SQL WHERE condition
      //

      int mode = 0; // 0->new query
      if (currentWords[0].equalsIgnoreCase("and") ||
          currentWords[0].equalsIgnoreCase("add"))
      {
         if (currentWords.length < 3)   mode=1;
         else                           mode=2;
      }


      System.out.println("parse(" + s + "): number of words in sentence=" +
			 currentWords.length);
      currentWordIndex=0;

      if (mode==0) {
        tableName=null;
        time_after=null;
        time_before=null;
        num_temp_col_names = 0;
        currentAction=NO_OP;
        displayColumnName = "*";
        searchColumnName = null;
        searchString="*";
        searchColumnName2 = null;
        searchString2=null;
        searchColumnName3 = null;
        searchString3=null;
      }  else if (mode==1) {
        System.out.println("processing 'add/and <column name>'");
        currentWordIndex++;
        String cname = eatColumnName(null);
        if (cname != null) {
          displayColumnName = displayColumnName + ", " + cname;
          return;
        }
      } else if (mode==2) {
         System.out.println("processing 'and/add <condition>'");
         currentWordIndex++;
         String cname1 = eatColumnName(null);
         if (cname1 != null) {
            System.out.println("   cname1=" + cname1);
            // look for a condition:
            if (eatWord(equals_is)) {
              System.out.println("  equals_is matched **");
              if (currentWordIndex < currentWords.length) {
                searchColumnName2 = cname1;
                searchString2 = currentWords[currentWordIndex];
                return;
             }
           }
         }
         return;
      }

      if (eatWord(show) == false) return;
      eatWord(noise1); // optional; also check for column names
      displayColumnName = eatColumnName(null);

      // check for more column names of the form:
      //   <cn>, <cn>, and <cn>
      // NOTE: "," chars are already removed.

      eatWord(and);
      String temp = eatColumnName(null);
      if (temp!=null) {
        displayColumnName = displayColumnName + ", " + temp;
        eatWord(and);
        temp = eatColumnName(null);
        if (temp!=null) {
            displayColumnName = displayColumnName + ", " + temp;
        }
      }

      if (displayColumnName==null) displayColumnName="*";
      eatWord(where);   // WHERE
      searchColumnName = eatColumnName(null);  // displayColumnName);
      System.out.println("searchColumnName=" + searchColumnName);
      currentAction=LIST;
      eatWord(is); // skip 'is'

      quantity=-999.0f;
      compareMode=NONE;

      if (eatWord(less)) {
        eatWord(than);  // skip 'than'
        String quan = currentWords[currentWordIndex];
        try {
           Float f = new Float(quan);
           quantity = f.floatValue();
           compareMode=LESS;
           currentWordIndex++;
           System.out.println("less than " + quantity);
        } catch (Exception e) { }
      }

      if (eatWord(more)) {
        eatWord(than);  // skip 'than'
        String quan = currentWords[currentWordIndex];
        try {
           Float f = new Float(quan);
           quantity = f.floatValue();
           compareMode=MORE;
           currentWordIndex++;
           System.out.println("more than " + quantity);
        } catch (Exception e) { }
      }

      if (eatWord(after)) {
         if (currentWords.length > currentWordIndex+2) {
            String test = currentWords[currentWordIndex] + " " +
                          currentWords[currentWordIndex+1] + " " +
                          currentWords[currentWordIndex+2];
            time_after = new SmartDate(test);
            if (time_after.valid()==false)  time_after=null;
            else currentWordIndex+=3;
         }
         if (time_after==null & currentWords.length > currentWordIndex+1) {
            String test = currentWords[currentWordIndex] + " " +
                          currentWords[currentWordIndex+1];
            time_after = new SmartDate(test);
            if (time_after.valid()==false)  time_after=null;
            else currentWordIndex+=2;
         }
         if (time_after==null & currentWords.length > currentWordIndex) {
            String test = currentWords[currentWordIndex];
            time_after = new SmartDate(test);
            if (time_after.valid()==false)  time_after=null;
            else currentWordIndex+=1;
         }
      }
      if (time_after!=null) {
        System.out.println("parsed 'after' time OK:");
        System.out.println("  year:  " + time_after.getYear());
        System.out.println("  month: " + time_after.getMonth());
        System.out.println("  day:   " + time_after.getDayOfMonth());
      }

      if (eatWord(before)) {
         if (currentWords.length > currentWordIndex+2) {
            String test = currentWords[currentWordIndex] + " " +
                          currentWords[currentWordIndex+1] + " " +
                          currentWords[currentWordIndex+2];
            time_before = new SmartDate(test);
            if (time_before.valid()==false)  time_before=null;
            else currentWordIndex+=3;
         }
         if (time_before==null & currentWords.length > currentWordIndex+1) {
            String test = currentWords[currentWordIndex] + " " +
                          currentWords[currentWordIndex+1];
            time_before = new SmartDate(test);
            if (time_before.valid()==false)  time_before=null;
            else currentWordIndex+=2;
         }
         if (time_before==null & currentWords.length > currentWordIndex) {
            String test = currentWords[currentWordIndex];
            time_before = new SmartDate(test);
            if (time_before.valid()==false)  time_before=null;
            else currentWordIndex+=1;
         }
      }
      if (time_before!=null) {
        System.out.println("parsed 'before' time OK:");
        System.out.println("  year:  " + time_before.getYear());
        System.out.println("  month: " + time_before.getMonth());
        System.out.println("  day:   " + time_before.getDayOfMonth());
      }

      conditionMode = 0;

      if (searchColumnName==null) return;
      if (eatWord(and)) {   // check for AND condition
         System.out.println("processing 'and/add <condition>'");
         String cname1 = eatColumnName(null);
         if (cname1 != null) {
            System.out.println("   cname1=" + cname1);
            // look for a condition:
            if (eatWord(equals_is)) {
              System.out.println("  equals_is matched **");
              if (currentWordIndex < currentWords.length) {
                searchColumnName2 = cname1;
                searchString2 = currentWords[currentWordIndex];
                conditionMode = AND;
             }
           }
         }
      }
      if (eatWord(or)) {   // check for OR condition
         System.out.println("processing 'and/add <condition>'");
         String cname1 = eatColumnName(null);
         if (cname1 != null) {
            System.out.println("   cname1=" + cname1);
            // look for a condition:
            if (eatWord(equals_is)) {
              System.out.println("  equals_is matched **");
              if (currentWordIndex < currentWords.length) {
                searchColumnName2 = cname1;
                searchString2 = currentWords[currentWordIndex];
                conditionMode = OR;
             }
           }
         }
      }

      if (eatWord(equals)==false) return;
      if (currentWordIndex<currentWords.length) {
        searchString=currentWords[currentWordIndex];
      }

    }

    public String getSQL() {
      if (currentAction==NO_OP) {
        System.out.println("getSQL(): currentAction is NO_OP!");
        return "";
      }
      if (currentAction==LIST) {
        // Start by making sure that the 'tableName' string does not
        // include any tables that are not referenced in column name
        // string:
        int index = displayColumnName.indexOf(".");
        if (index>-1) {
            tableName = displayColumnName.substring(0,index);
        }
        if (searchColumnName!=null) {
            index = searchColumnName.indexOf(".");
            if (index>-1) {
                searchColumnName = tableName + searchColumnName.substring(index);
            }
        }
        if (searchColumnName2!=null) {
            index = searchColumnName2.indexOf(".");
            if (index>-1) {
                searchColumnName2 = tableName + searchColumnName2.substring(index);
            }
        }
        if (searchColumnName3!=null) {
            index = searchColumnName3.indexOf(".");
            if (index>-1) {
                searchColumnName3 = tableName + searchColumnName3.substring(index);
            }
        }
        StringBuffer sb = new StringBuffer();
        sb.append("SELECT ");
        if (displayColumnName!=null) sb.append(displayColumnName);
        else                         sb.append("*");
        if (time_after!=null && time_after.valid()) {
          if (searchColumnName!=null) {
            sb.append(" FROM " + tableName + " WHERE " + searchColumnName +
                      " > #" + time_after.toString() + "#");
          } else {
            sb.append(" FROM " + tableName);
          }
        } else if (time_before!=null && time_before.valid()) {
          if (searchColumnName!=null) {
            sb.append(" FROM " + tableName + " WHERE " + searchColumnName +
                      " < #" + time_before.toString() + "#");
          } else {
            sb.append(" FROM " + tableName);
          }
        } else if (compareMode!=NONE && quantity!=-999.0f) {
          if (compareMode==LESS) {
            if (searchColumnName!=null) {
              sb.append(" FROM " + tableName + " WHERE " + searchColumnName +
                        " < " + quantity);
            } else {
              sb.append(" FROM " + tableName);
            }
          } else { // MORE
            if (searchColumnName!=null) {
              sb.append(" FROM " + tableName + " WHERE " + searchColumnName +
                        " > " + quantity);
            } else {
              sb.append(" FROM " + tableName);
            }
          }
        } else {
          if (searchColumnName!=null) {
            sb.append(" FROM " + tableName + " WHERE " + searchColumnName +
                      " = " + quoteLiteral(searchString));
          } else {
            sb.append(" FROM " + tableName);
          }
        }
        if (searchString2!=null && conditionMode==AND) {
          sb.append(" AND " + searchColumnName2 + " = " +
                      quoteLiteral(searchString2));
        }
        if (searchString2!=null && conditionMode==OR) {
          sb.append(" OR " + searchColumnName2 + " = " +
                      quoteLiteral(searchString2));
        }
        sb.append(" ;");
        return new String(sb);
      }
      return "";
    }

    public String toEnglish(String [] r, String [] syns, 
                            String [] origs, int num) {
      System.out.print("\ntoEnglish(");
      for (int i=0; i<r.length; i++) {
        System.out.println(r[i] + ", ");
      }
      System.out.println(", , " + num + ")");
      for (int i=0; i<num; i++) {
        System.out.println("  syns[" + i + "]=" + syns[i] +
          ", origs[" + i + "]=" + origs[i]);
      }
      for (int i=0; i<num_temp_col_names; i++) {
        System.out.println("  temp_col_names[" + i + "]=" +
          temp_col_names[i]);
      }
      StringBuffer sb = new StringBuffer("The value of ");
      for (int i=0; i<r.length; i++) {
        String s = temp_col_names[i];
        // check for synonym substitution:
        for (int j=0; j<num; j++) {
          if (s.equalsIgnoreCase(syns[j])) {
            s = origs[j];
          }
        }
        sb.append(s + " is " + r[i]);
        if (i<r.length - 2) sb.append(", ");
        if (i==r.length- 2) sb.append(" and ");
      }
      sb.append(".");
      return new String(sb);
    }

    // String constants need ' ' marks, while numbers can
    // not have surrounding ' ' marks:
    private String quoteLiteral(String s) {
      if (s.startsWith("0")) return s;
      if (s.startsWith("1")) return s;
      if (s.startsWith("2")) return s;
      if (s.startsWith("3")) return s;
      if (s.startsWith("4")) return s;
      if (s.startsWith("5")) return s;
      if (s.startsWith("6")) return s;
      if (s.startsWith("7")) return s;
      if (s.startsWith("8")) return s;
      if (s.startsWith("9")) return s;
      return "'" + s + "'";
    }


    // Semantic action codes:
    public final static int NO_OP=0;
    public final static int LIST=1;
    public final static int DELETE_DATA=2;
    public final static int MODIFY_DATA=3;
    public final static int NONE=0;
    public final static int LESS=1;
    public final static int MORE=2;
            
    private String show[] = {"show", "list", "display"};
    private String noise1[] = {"data", "info", "information", "any", "all", "everything"};
    private String where[] = {"where"};
    private String equals[] = {"equals", "contains"};
    private String is[] = {"is"};
    private String equals_is[] = {"equals", "contains", "is"};
    private String after[] = {"after"};
    private String before[] = {"before"};
    private String and[] = {"and"};
    private String or[] = {"or"};
    private String less[] = {"less", "smaller"};
    private String more[] = {"more", "greater", "larger"};
    private String than[] = {"than"};

    private int currentAction=NO_OP;
    private String displayColumnName = null;
    private String searchColumnName = null;
    private String searchString=null;
    private String searchColumnName2 = null;
    private String searchString2=null;
    private String searchColumnName3 = null;
    private String searchString3=null;
    private int conditionMode = 0;
    private String tableName="";

    private SmartDate time_after = null;
    private SmartDate time_before= null;

    private float quantity=-999.0f;
    private int compareMode=NONE;

    public int getAction() {
        // return the current action code
        return NO_OP;
    }

    private String currentWords[];
    private int currentWordIndex=0;

    private boolean eatWord(String s[]) {
      if (currentWordIndex>=currentWords.length) {
        System.out.println("eatWord(): no more words to test!");
        return false;
      }
      System.out.println("eatWord(): testing " + currentWords[currentWordIndex]);
      for (int i=0; i<s.length; i++) {
        if (currentWords[currentWordIndex].equalsIgnoreCase(s[i])) {
          System.out.println("   eatWord processed: " + currentWords[currentWordIndex]);
          currentWordIndex++;
          return true;
        }
      }
      return false;
    }

    private String eatColumnName(String current_column_name) {
      System.out.println("eatColumnName(" + current_column_name + ")");
      if (currentWordIndex>=currentWords.length) {
        System.out.println("eatColumnName(" + current_column_name + "): no more words to test");
        return null;
      }
      // Check for a column name of the form <Table name>.<column name>:
      if (current_column_name!=null) {
        int index = current_column_name.indexOf(".");
        if (index>-1) {
          current_column_name = current_column_name.substring(index+1);
          System.out.println("*** new column name (table name removed): " +
                            current_column_name);
        }
      }
      String ret_col_name=null;
      for (int i=0; i<dbinfo.num_tables; i++) {
        for (int j=0; j<dbinfo.columnNames[i].length; j++) {
         if (currentWordIndex>=currentWords.length) break;
          //System.out.println("currentWords[" + currentWordIndex + "]=" + currentWords[currentWordIndex]);
          //System.out.println("dbinfo.columnNames[" + i + "][" + j + "]=" + dbinfo.columnNames[i][j]);
          if (currentWords[currentWordIndex].equalsIgnoreCase(dbinfo.columnNames[i][j])) {
            if (current_column_name!=null &&
                current_column_name.indexOf(dbinfo.columnNames[i][j]) >-1)
                continue;
            System.out.println("eatColumnName(): found: " +
                               dbinfo.columnNames[i][j]);
            currentWordIndex++;

            temp_col_names[num_temp_col_names++] = dbinfo.columnNames[i][j];

            if (tableName==null) {
                tableName=dbinfo.name[i];
            } else {
              if (tableName.equalsIgnoreCase(dbinfo.name[i])==false) {
                tableName=tableName + ", " + dbinfo.name[i];
              }
            }
            if (ret_col_name==null) {
                ret_col_name=dbinfo.name[i] + "." + dbinfo.columnNames[i][j];
            } else {
              if (ret_col_name.equalsIgnoreCase(dbinfo.columnNames[i][j])==false) {
                ret_col_name=ret_col_name + ", " +
                   dbinfo.name[i] + "." + dbinfo.columnNames[i][j];
              }
            }
            System.out.println("  setting table name=" + tableName);
          }
        }
      }
      if (ret_col_name!=null)
          System.out.println("** eatColumnName(" + current_column_name + "), ret_col_name=" + ret_col_name);
      return ret_col_name;
    }

    private DBInfo dbinfo;
    private String [] temp_col_names = new String[11];
    private int num_temp_col_names = 0;

}
