// mwa.products.SmartDate.java
//
// Copyright 1997, Mark Watson.
//
// This software is Copyright 1998 by Mark Watson.
// This software may be freely used for any purpose
// and may be modified for any purpose as long as
// you agree to the following:
//
// 1. Any bug fixes or enhancements that you make must be emailed
//    back to Mark Watson (markw@markwatson.com) for possible
//    inclusion in the source code base that is available at
//    the web site www.markwatson.com (credit will be given
//    to contributers).
//
// 2. This copyright notice must be kept with the source code
//    and duplicated if you distribute a modified verison of
//    this software in binary form.
//
// 3. You will not re-distribute the source code. Rather, any
//    improvements to this program will be returned to Mark
//    Watson so that there is always one source base at
//    the web site www.markwatson.com (you may freely
//    distribute compiled derived versions of this program
//    without any restrictions as long as this entire
//    copyright notice is reproduced with the application
//    (e.g., in the 'help' or 'about' dialog box).
//
// 4. This software may not be used for any illegal purpose.
//
// 5. You take all responsibility for the use of this software:
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// Open Source contributers:
//
//   Name                        Email (optional)
//   ----                        ----------------
//



package com.markwatson.nlbean;

import java.text.*;
import java.util.*;
import java.io.*;

// Utility class to handle dates in fairly arbitrary natural language formats:
class SmartDate implements Serializable {
  private java.util.Date date = null;
  private Calendar calendar = null;
  public SmartDate(String s) {
    System.out.println("\n*****  SmartDate(" + s + ")");
    // Try to create a 'SimpleDate' parse string for this string:
    SimpleDateFormat sdf;
    try {
      if (date == null) {
	sdf = new SimpleDateFormat("MMM d yyyy");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("yyyy MM dd hh mm ss");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("yyyy mm dd");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("yy mm dd");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("EEE MMM d yyyy");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("EEE MMM d ''yy");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("yyy-mm-dd hh:mm:ss");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
      if (date == null) {
	sdf = new SimpleDateFormat("MMM d ''yy");
	try {
	  date = sdf.parse(s);
	} catch (Exception pe) { date = null; }
      }
    } catch (Exception e) {
      System.out.println("Parse error: " + s + ", " + e);
    }
    if (date!=null) {
      calendar = GregorianCalendar.getInstance();
      calendar.setTime(date);
      //System.out.print("ERA: " + calendar.get(Calendar.ERA) + ". ");
      System.out.print("YEAR: " + calendar.get(Calendar.YEAR) + ". ");
      System.out.print("MONTH: " + calendar.get(Calendar.MONTH) + ". ");
      //System.out.print("WEEK_OF_YEAR: " + calendar.get(Calendar.WEEK_OF_YEAR) + ". ");
      //System.out.print("WEEK_OF_MONTH: " + calendar.get(Calendar.WEEK_OF_MONTH) + ". ");
      System.out.print("DATE: " + calendar.get(Calendar.DATE) + ". ");
      System.out.print("DAY_OF_MONTH: " + calendar.get(Calendar.DAY_OF_MONTH) + ". ");
      //System.out.print("DAY_OF_YEAR: " + calendar.get(Calendar.DAY_OF_YEAR) + ". ");
      //System.out.print("DAY_OF_WEEK: " + calendar.get(Calendar.DAY_OF_WEEK) + ". ");
      //System.out.print("DAY_OF_WEEK_IN_MONTH: "
      //                   + calendar.get(Calendar.DAY_OF_WEEK_IN_MONTH) + ". ");
      //System.out.print("AM_PM: " + calendar.get(Calendar.AM_PM) + ". ");
      //System.out.print("HOUR: " + calendar.get(Calendar.HOUR) + ". ");
      //System.out.print("HOUR_OF_DAY: " + calendar.get(Calendar.HOUR_OF_DAY) + ". ");
      //System.out.print("MINUTE: " + calendar.get(Calendar.MINUTE) + ". ");
      //System.out.print("SECOND: " + calendar.get(Calendar.SECOND) + ". ");
      //System.out.print("MILLISECOND: " + calendar.get(Calendar.MILLISECOND) + ". ");
      System.out.println("");
    } else {
      System.out.println("Parse error: " + s);
    }
  }
  public int getYear() {
    if (date!=null && calendar!=null) {
      return calendar.get(Calendar.YEAR);
    }
    return 0;
  }
  public int getMonth() {
    if (date!=null && calendar!=null) {
      return calendar.get(Calendar.MONTH);
    }
    return 0;
  }
  public int getDayOfMonth() {
    if (date!=null && calendar!=null) {
      return calendar.get(Calendar.DAY_OF_MONTH);
    }
    return 0;
  }
  public long getMilliseconds() {
    if (date!=null && calendar!=null) {
      return calendar.get(Calendar.MILLISECOND);
    }
    return 0;
  }
  public boolean valid() {
    if (calendar==null) return false;
    if (calendar.get(Calendar.MILLISECOND)!=0) return true;
    if (calendar.get(Calendar.YEAR)!=0) return true;
    if (calendar.get(Calendar.DAY_OF_MONTH)!=0) return true;
    if (calendar.get(Calendar.DATE)!=0) return true;
    return false;
  }
  public String toString() {
    if (valid()==false) return "<not valid date>";
    //return getMonth() + "/" + getDayOfMonth()+1 + "/" + getYear();
    int month = getMonth() + 1;
    return getYear() + "-" + month + "-" + getDayOfMonth();
    //SimpleDateFormat formatter
    //  = new SimpleDateFormat ("mm/dd/yyyy");
    //return formatter.format(date);
  }
}
